@@ -75,6 +75,12 @@ enum {
 	HCI_RAW,
 };
 
+/* LE address type */
+enum {
+	LE_PUBLIC_ADDRESS = 0x00,
+	LE_RANDOM_ADDRESS = 0x01
+};
+
 /* HCI ioctl defines */
 #define HCIDEVUP	_IOW('H', 201, int)
 #define HCIDEVDOWN	_IOW('H', 202, int)
@@ -196,6 +202,7 @@ enum {
 #define HCI_HOST_BUSY_PAIRING			0x38
 
 /* ACL flags */
+#define ACL_START_NO_FLUSH	0x00
 #define ACL_CONT		0x01
 #define ACL_START		0x02
 #define ACL_ACTIVE_BCAST	0x04
@@ -784,16 +791,18 @@ typedef struct {
 } __attribute__ ((packed)) delete_stored_link_key_rp;
 #define DELETE_STORED_LINK_KEY_RP_SIZE 3
 
+#define HCI_MAX_NAME_LENGTH		248
+
 #define OCF_CHANGE_LOCAL_NAME		0x0013
 typedef struct {
-	uint8_t		name[248];
+	uint8_t		name[HCI_MAX_NAME_LENGTH];
 } __attribute__ ((packed)) change_local_name_cp;
 #define CHANGE_LOCAL_NAME_CP_SIZE 248
 
 #define OCF_READ_LOCAL_NAME		0x0014
 typedef struct {
 	uint8_t		status;
-	uint8_t		name[248];
+	uint8_t		name[HCI_MAX_NAME_LENGTH];
 } __attribute__ ((packed)) read_local_name_rp;
 #define READ_LOCAL_NAME_RP_SIZE 249
 
@@ -1064,18 +1073,20 @@ typedef struct {
 } __attribute__ ((packed)) write_afh_mode_rp;
 #define WRITE_AFH_MODE_RP_SIZE 1
 
+#define HCI_MAX_EIR_LENGTH		240
+
 #define OCF_READ_EXT_INQUIRY_RESPONSE	0x0051
 typedef struct {
 	uint8_t		status;
 	uint8_t		fec;
-	uint8_t		data[240];
+	uint8_t		data[HCI_MAX_EIR_LENGTH];
 } __attribute__ ((packed)) read_ext_inquiry_response_rp;
 #define READ_EXT_INQUIRY_RESPONSE_RP_SIZE 242
 
 #define OCF_WRITE_EXT_INQUIRY_RESPONSE	0x0052
 typedef struct {
 	uint8_t		fec;
-	uint8_t		data[240];
+	uint8_t		data[HCI_MAX_EIR_LENGTH];
 } __attribute__ ((packed)) write_ext_inquiry_response_cp;
 #define WRITE_EXT_INQUIRY_RESPONSE_CP_SIZE 241
 typedef struct {
@@ -1379,7 +1390,7 @@ typedef struct {
 	uint8_t		status;
 	uint8_t		handle;
 	uint16_t	length;
-	uint8_t		fragment[248];
+	uint8_t		fragment[HCI_MAX_NAME_LENGTH];
 } __attribute__ ((packed)) read_local_amp_assoc_rp;
 #define READ_LOCAL_AMP_ASSOC_RP_SIZE 252
 
@@ -1388,7 +1399,7 @@ typedef struct {
 	uint8_t		handle;
 	uint16_t	length_so_far;
 	uint16_t	assoc_length;
-	uint8_t		fragment[248];
+	uint8_t		fragment[HCI_MAX_NAME_LENGTH];
 } __attribute__ ((packed)) write_remote_amp_assoc_cp;
 #define WRITE_REMOTE_AMP_ASSOC_CP_SIZE 253
 typedef struct {
@@ -1715,7 +1726,7 @@ typedef struct {
 typedef struct {
 	uint8_t		status;
 	bdaddr_t	bdaddr;
-	uint8_t		name[248];
+	uint8_t		name[HCI_MAX_NAME_LENGTH];
 } __attribute__ ((packed)) evt_remote_name_req_complete;
 #define EVT_REMOTE_NAME_REQ_COMPLETE_SIZE 255
 
@@ -1976,7 +1987,7 @@ typedef struct {
 	uint8_t		dev_class[3];
 	uint16_t	clock_offset;
 	int8_t		rssi;
-	uint8_t		data[240];
+	uint8_t		data[HCI_MAX_EIR_LENGTH];
 } __attribute__ ((packed)) extended_inquiry_info;
 #define EXTENDED_INQUIRY_INFO_SIZE 254
 
@@ -2253,9 +2264,13 @@ typedef struct {
 struct sockaddr_hci {
 	sa_family_t	hci_family;
 	unsigned short	hci_dev;
+	unsigned short  hci_channel;
 };
 #define HCI_DEV_NONE	0xffff
 
+#define HCI_CHANNEL_RAW		0
+#define HCI_CHANNEL_CONTROL	1
+
 struct hci_filter {
 	uint32_t type_mask;
 	uint32_t event_mask[2];
